import 'package:masamune/masamune.dart';
import 'package:masamune_module/masamune_module.dart';

part 'detail.m.dart';

@module
@immutable
class DetailModule extends PageModule {
  const DetailModule({
    bool enabled = true,
    String? title,
    this.routePath = "detail",
    required this.queryPath,
    this.commentQueryPath = "comment",
    this.nameKey = Const.name,
    this.textKey = Const.text,
    this.imageKey = Const.image,
    this.iconKey = Const.icon,
    this.timeKey = Const.time,
    this.userKey = Const.user,
    this.likeCountKey = "likeCount",
    this.userPath = Const.user,
    this.enableBookmark = true,
    this.enableComment = true,
    this.enableLike = true,
    this.bookmarkPath = "bookmark",
    this.likePath = "like",
    this.expandedHeight = 240,
    Permission permission = const Permission(),
    RerouteConfig? rerouteConfig,
    this.home,
  }) : super(
          enabled: enabled,
          title: title,
          permission: permission,
          rerouteConfig: rerouteConfig,
        );

  @override
  Map<String, RouteConfig>? get routeSettings {
    if (!enabled) {
      return const {};
    }

    final route = {
      "/$routePath/{detail_id}":
          RouteConfig((_) => home ?? DetailModuleHome(this)),
    };
    return route;
  }

  // Page settings.
  final Widget? home;

  /// Keys.
  final String nameKey;
  final String iconKey;
  final String textKey;
  final String imageKey;
  final String timeKey;
  final String userKey;
  final String likeCountKey;

  /// ツールバーの高さ
  final double expandedHeight;

  /// Route path.
  final String routePath;

  /// Query path.
  final String queryPath;
  final String userPath;
  final String likePath;
  final String bookmarkPath;
  final String commentQueryPath;

  final bool enableLike;
  final bool enableBookmark;
  final bool enableComment;

  @override
  DetailModule? fromMap(DynamicMap map) => _$DetailModuleFromMap(map, this);

  @override
  DynamicMap toMap() => _$DetailModuleToMap(this);
}

class DetailModuleHome extends PageScopedWidget {
  const DetailModuleHome(this.config);
  final DetailModule config;

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    // Please describe reference.
    final detailId = context.get("detail_id", "");
    final detail = ref.watchDocumentModel("${config.queryPath}/$detailId");
    final name = detail.get(config.nameKey, "");
    final text = detail.get(config.textKey, "");
    final image = detail.get(config.imageKey, "");
    final time = detail.getAsDateTime(config.timeKey);
    final count = detail.get(config.likeCountKey, 0);
    final comment = ref.watchCollectionModel(
      ModelQuery(
        "${config.queryPath}/$detailId/${config.commentQueryPath}",
        order: ModelQueryOrder.desc,
        orderBy: config.timeKey,
      ).value,
    );
    final users = ref.watchCollectionModel(
      ModelQuery(
        config.userPath,
        key: "uid",
        whereIn: comment.map((e) => e.get(config.userKey, "")).distinct(),
      ).value,
    );
    final commentWithUser = comment.setWhere(
      users,
      test: (o, a) => o.get(config.userKey, "") == a.uid,
      apply: (o, a) =>
          o.merge(a, convertKeys: (key) => "${config.userKey}$key"),
      orElse: (o) => o,
    );
    final like = ref
        .watchDocumentModel(
            "${config.userPath}/${context.model?.userId}/${config.likePath}/$detailId")
        .isNotEmpty;
    final bookmark = ref
        .watchDocumentModel(
            "${config.userPath}/${context.model?.userId}/${config.bookmarkPath}/$detailId")
        .isNotEmpty;

    // Please describe the Widget.
    return UIScaffold(
      appBar: UIModernDetailAppBar(
        title: Text(name),
        designType: DesignType.modern,
        expandedHeight: config.expandedHeight,
        backgroundImage: image.isNotEmpty ? NetworkOrAsset.image(image) : null,
        bottomActions: [Text(time.format("yyyy/MM/dd HH:mm"))],
      ),
      body: ListBuilder<DynamicMap>(
        padding: const EdgeInsets.all(0),
        source: commentWithUser.toList(),
        top: [
          Indent(
            padding: const EdgeInsets.fromLTRB(16, 24, 16, 16),
            children: [
              Text(name, style: context.theme.textTheme.headline5),
              const Space.height(8),
              Text(text),
              const Space.height(8),
              Row(
                mainAxisSize: MainAxisSize.max,
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: [
                  TextButton.icon(
                    style: TextButton.styleFrom(
                      primary: like
                          ? context.theme.primaryColor
                          : context.theme.disabledColor,
                    ),
                    icon: const Icon(Icons.thumb_up),
                    onPressed: () {
                      final counter = context.model?.incrementCounter(
                        collectionPath:
                            "${config.queryPath}/$detailId/${config.likePath}",
                        linkedCollectionPath:
                            "${config.userPath}/${context.model!.userId}/${config.likePath}",
                      );
                      if (like) {
                        counter?.remove(context.model!.userId,
                            linkId: detail.uid);
                      } else {
                        counter?.append(context.model!.userId,
                            linkId: detail.uid);
                      }
                    },
                    label: Text(
                      count.toString(),
                      style: const TextStyle(fontSize: 18),
                    ),
                  ),
                  IconButton(
                    color: context.theme.disabledColor,
                    icon: const Icon(Icons.comment),
                    onPressed: () {},
                  ),
                  IconButton(
                    color: context.theme.disabledColor,
                    icon: const Icon(Icons.share),
                    onPressed: () {},
                  ),
                  IconButton(
                    color: bookmark
                        ? context.theme.primaryColor
                        : context.theme.disabledColor,
                    icon: const Icon(Icons.bookmark),
                    onPressed: () {
                      final counter = context.model?.incrementCounter(
                        collectionPath:
                            "${config.queryPath}/$detailId/${config.bookmarkPath}",
                        linkedCollectionPath:
                            "${config.userPath}/${context.model!.userId}/${config.bookmarkPath}",
                      );
                      if (like) {
                        counter?.remove(context.model!.userId,
                            linkId: detail.uid);
                      } else {
                        counter?.append(context.model!.userId,
                            linkId: detail.uid);
                      }
                    },
                  )
                ],
              ),
            ],
          ),
          const Divid(),
        ],
        builder: (context, item, index) {
          return [
            CommentTile(
              image: NetworkOrAsset.image(
                item.get("${config.userKey}${config.iconKey}", ""),
              ),
              name: item.get("${config.userKey}${config.nameKey}", ""),
              text: item.get(config.textKey, ""),
              date: item.getAsDateTime(config.timeKey),
            ),
          ];
        },
      ),
    );
  }
}
